from __future__ import annotations

import logging
import random
import typing
from datetime import timedelta
from typing import Optional, Union
from uuid import UUID

import isodate
from flyteidl.core import artifact_id_pb2 as art_id
from flytekit.core.artifact import Artifact as FlytekitArtifact
from flytekit.core.artifact import (
    ArtifactQuery,
    ArtifactSerializationHandler,
    Partition,
    Partitions,
    Serializer,
    TimePartition,
)
from flytekit.models.literals import Literal
from flytekit.models.types import LiteralType

from unionai.internal.artifacts import artifacts_pb2

if typing.TYPE_CHECKING:
    from unionai.remote import UnionRemote

logger = logging.getLogger("unionai.artifacts")


class Artifact(FlytekitArtifact):
    """
    This is a wrapper around the Flytekit Artifact class.

    This Python class has two purposes - as a Python representation of a materialized Artifact,
    and as a way for users to specify that tasks/workflows create Artifacts and the manner
    in which they are created.

    Use one as input to workflow (only workflow for now)
    df_artifact = Artifact.get("flyte://a1")
    remote.execute(wf, inputs={"a": df_artifact})

    Note that Python fields will be missing when retrieved from the service.
    """

    def __init__(
        self,
        *args,
        project: Optional[str] = None,
        domain: Optional[str] = None,
        name: Optional[str] = None,
        version: Optional[str] = None,
        time_partitioned: bool = False,
        time_partition: Optional[TimePartition] = None,
        partition_keys: Optional[typing.List[str]] = None,
        partitions: Optional[Union[Partitions, typing.Dict[str, str]]] = None,
        python_val: Optional[typing.Any] = None,
        python_type: Optional[typing.Type] = None,
        literal: Optional[Literal] = None,
        literal_type: Optional[LiteralType] = None,
        short_description: Optional[str] = None,
        source: Optional[artifacts_pb2.ArtifactSource] = None,
        **kwargs,
    ):
        """
        :param project: Should not be directly user provided, the project/domain will come from the project/domain of
           the execution that produced the output. These values will be filled in automatically when retrieving however.
        :param domain: See above.
        :param name: The name of the Artifact. This should be user provided.
        :param version: Version of the Artifact, typically the execution ID, plus some additional entropy.
           Not user provided.
        :param time_partitioned: Whether or not this Artifact will have a time partition.
        :param partition_keys: This is a list of keys that will be used to partition the Artifact. These are not the
           values. Values are set via a () on the artifact and will end up in the partition_values field.
        :param partitions: This is a dictionary of partition keys to values.
        :param python_val: The Python value.
        :param python_type: The Python type.
        """
        self.python_val = python_val
        self.python_type = python_type
        self.literal = literal
        self.literal_type = literal_type
        self.short_description = short_description
        self.source = source
        super().__init__(
            *args,
            project=project,
            domain=domain,
            name=name,
            version=version,
            time_partitioned=time_partitioned,
            time_partition=time_partition,
            partition_keys=partition_keys,
            partitions=partitions,
            **kwargs,
        )

    @classmethod
    def initialize(
        cls,
        python_val: typing.Any,
        python_type: typing.Type,
        name: Optional[str] = None,
        literal_type: Optional[LiteralType] = None,
        tags: Optional[typing.List[str]] = None,
    ) -> Artifact:
        """
        Use this for when you have a Python value you want to get an Artifact object out of.

        This function readies an Artifact for creation, it doesn't actually create it just yet since this is a
        network-less call. You will need to persist it with a FlyteRemote instance:
            remote.create_artifact(Artifact.initialize(...))

        Artifact.initialize("/path/to/file", tags={"tag1": "val1"})
        Artifact.initialize("/path/to/parquet", type=pd.DataFrame, tags=["0.1.0"])

        What's set here is everything that isn't set by the server. What is set by the server?
        - name, version, if not set by user.
        - uri
        Set by remote
        - project, domain
        """
        # Create the artifact object
        return Artifact(
            python_val=python_val,
            python_type=python_type,
            literal_type=literal_type,
            tags=tags,
            name=name,
        )

    @classmethod
    def from_flyte_idl(cls, pb2: artifacts_pb2.Artifact) -> Artifact:
        """
        Converts the IDL representation to this object.
        """
        a = Artifact(
            project=pb2.artifact_id.artifact_key.project,
            domain=pb2.artifact_id.artifact_key.domain,
            name=pb2.artifact_id.artifact_key.name,
            version=pb2.artifact_id.version,
            literal_type=LiteralType.from_flyte_idl(pb2.spec.type),
            literal=Literal.from_flyte_idl(pb2.spec.value),
            source=pb2.source,
        )
        if pb2.artifact_id.HasField("partitions"):
            if len(pb2.artifact_id.partitions.value) > 0:
                # static values should be the only ones set since currently we don't from_flyte_idl
                # anything that's not a materialized artifact.
                # if TIME_PARTITION in pb2.artifact_id.partitions.value:
                #     a._time_partition = TimePartition(pb2.artifact_id.partitions.value[TIME_PARTITION].static_value)
                #     a._time_partition.reference_artifact = a

                a._partitions = Partitions(
                    partitions={
                        k: Partition(value=v, name=k)
                        for k, v in pb2.artifact_id.partitions.value.items()
                        # if k != TIME_PARTITION
                    }
                )
                a.partitions.reference_artifact = a
        if pb2.artifact_id.HasField("time_partition"):
            ts = pb2.artifact_id.time_partition.value.time_value
            dt = ts.ToDatetime()
            a._time_partition = TimePartition(dt)
            a._time_partition.reference_artifact = a

        return a

    def __str__(self):
        tp_str = f"  time partition={self.time_partition}\n" if self.time_partitioned else ""
        return (
            f"Artifact: project={self.project}, domain={self.domain}, name={self.name}, version={self.version}\n"
            f"  name={self.name}\n"
            f"  partitions={self.partitions}\n"
            f"{tp_str}"
            f"  literal_type="
            f"{self.literal_type}, "
            f"literal={self.literal})"
        )

    @classmethod
    def get(
        cls,
        uri: Optional[str],
        artifact_id: Optional[art_id.ArtifactID],
        remote: UnionRemote,
        get_details: bool = False,
    ) -> Optional[Artifact]:
        """
        Use one locally. This retrieves the Literal.
        a = remote.get("flyte://blah")
        a = Artifact.get("flyte://blah", remote, tag="latest")
        u = union.get("union://blah")
        """
        return remote.get_artifact(uri=uri, artifact_id=artifact_id, get_details=get_details)

    @staticmethod
    def to_create_request(a: Artifact) -> artifacts_pb2.CreateArtifactRequest:
        if not a.project or not a.domain:
            raise ValueError("Project and domain are required to create an artifact")
        name = a.name or UUID(int=random.getrandbits(128)).hex
        ak = art_id.ArtifactKey(project=a.project, domain=a.domain, name=name)

        spec = artifacts_pb2.ArtifactSpec(
            value=a.literal,
            type=a.literal_type,
        )
        partitions = Serializer.partitions_to_idl(a.partitions)

        tp = None
        if a._time_partition:
            tv = a.time_partition.value.time_value
            if not tv:
                raise Exception("missing time value")
            tp = a.time_partition.value.time_value

        return artifacts_pb2.CreateArtifactRequest(
            artifact_key=ak, spec=spec, partitions=partitions, time_partition_value=tp
        )


class UnionArtifactSerializationHandler(ArtifactSerializationHandler):
    def partitions_to_idl(self, p: Optional[Partitions], **kwargs) -> Optional[art_id.Partitions]:
        if not p or not p.partitions:
            return None

        bindings: Optional[typing.List[Artifact]] = kwargs.get("bindings", None)
        if bindings and len(bindings) > 0:
            return self.get_partitions_idl_with_binding(p, bindings)

        pp = {}
        for k, v in p.partitions.items():
            if v.value is None:
                # This should only happen when serializing for triggers
                # Probably indicative of something in the data model that can be fixed
                # down the road.
                pp[k] = art_id.LabelValue(static_value="")
            else:
                pp[k] = v.value
        return art_id.Partitions(value=pp)

    @staticmethod
    def get_partitions_idl_with_binding(p: Partitions, bindings: typing.List[Artifact] = None) -> art_id.Partitions:
        pp = {}
        # First create partition requirements for all the partitions
        if p.reference_artifact and p.reference_artifact in bindings:
            idx = bindings.index(p.reference_artifact)
            triggering_artifact = bindings[idx]
            if triggering_artifact.partition_keys:
                for k in triggering_artifact.partition_keys:
                    pp[k] = art_id.LabelValue(
                        triggered_binding=art_id.ArtifactBindingData(
                            index=idx,
                            partition_key=k,
                        )
                    )

        for k, v in p.partitions.items():
            if not v.reference_artifact or (
                v.reference_artifact
                and v.reference_artifact is p.reference_artifact
                and v.reference_artifact not in bindings
            ):
                # consider changing condition to just check for static value
                pp[k] = art_id.LabelValue(static_value=v.value.static_value)
            elif v.reference_artifact in bindings:
                # This line here is why the PartitionValue object has a name field.
                # We might bind to a partition key that's a different name than the k here.
                pp[k] = art_id.LabelValue(
                    triggered_binding=art_id.ArtifactBindingData(
                        index=bindings.index(v.reference_artifact),
                        partition_key=v.name,
                    )
                )
            else:
                raise ValueError(f"Partition has unhandled reference artifact {v.reference_artifact}")

        return art_id.Partitions(value=pp)

    def time_partition_to_idl(self, tp: TimePartition, **kwargs) -> Optional[art_id.TimePartition]:
        bindings: Optional[typing.List[Artifact]] = kwargs.get("bindings", None)

        if bindings and len(bindings) > 0:
            return self.get_time_partition_idl_with_binding(tp, bindings)

        if not tp.value:
            # This is only for triggers - the backend needs to know of the existence of a time partition
            return art_id.TimePartition()

        return art_id.TimePartition(value=tp.value)

    @staticmethod
    def get_time_partition_idl_with_binding(tp: TimePartition, bindings: typing.List[Artifact]) -> art_id.TimePartition:
        if not tp.reference_artifact or (tp.reference_artifact and tp.reference_artifact not in bindings):
            # basically if there's no reference artifact, or if the reference artifact isn't
            # in the list of triggers, then treat it like normal.
            return art_id.TimePartition(value=tp.value)
        elif tp.reference_artifact in bindings:
            idx = bindings.index(tp.reference_artifact)
            transform = None
            if tp.op and tp.other and isinstance(tp.other, timedelta):
                transform = str(tp.op) + isodate.duration_isoformat(tp.other)
            lv = art_id.LabelValue(
                triggered_binding=art_id.ArtifactBindingData(
                    index=idx,
                    bind_to_time_partition=True,
                    transform=transform,
                )
            )
            return art_id.TimePartition(value=lv)
        # investigate if this happens, if not, remove. else
        logger.warning(f"Investigate - time partition in trigger with unhandled reference artifact {tp}")
        raise ValueError("Time partition reference artifact not found in ")

    def artifact_query_to_idl(self, aq: ArtifactQuery, **kwargs) -> art_id.ArtifactQuery:
        bindings: Optional[typing.List[Artifact]] = kwargs.get("bindings", None)

        ak = art_id.ArtifactKey(
            name=aq.name,
            project=aq.project,
            domain=aq.domain,
        )

        p = Serializer.partitions_to_idl(aq.partitions, bindings=bindings)
        tp = None
        if aq.time_partition:
            tp = aq.time_partition.to_flyte_idl(bindings=bindings)

        i = art_id.ArtifactID(
            artifact_key=ak,
            partitions=p,
            time_partition=tp,
        )

        aq = art_id.ArtifactQuery(
            artifact_id=i,
        )

        return aq


logger.debug("Overriding artifact serializer with Union's")
Serializer.register_serializer(UnionArtifactSerializationHandler())



import asyncio
import traceback

import aiofiles
import fsspec
import grpc
from fsspec.asyn import AsyncFileSystem

from unionai.filesystems._async_utils import mirror_sync_methods
from unionai.filesystems._endpoint import _create_channel
from unionai.internal.common import list_pb2
from unionai.internal.objectstore.definition_pb2 import Key, Metadata, Object
from unionai.internal.objectstore.metadata_service_pb2_grpc import MetadataStoreServiceStub
from unionai.internal.objectstore.payload_pb2 import (
    DeleteRequest,
    GetRequest,
    GetResponse,
    HeadRequest,
    HeadResponse,
    ListRequest,
    ListResponse,
    PutRequest,
)


class AsyncUnionMetaFS(AsyncFileSystem):
    mirror_sync_methods = False

    def _prefix_path(self, path):
        if path.startswith(f"{self.protocol}://"):
            return path
        return f"{self.protocol}://{path}"

    def __init__(self, *args, **kwargs):
        channel = _create_channel()
        self._client = MetadataStoreServiceStub(channel)
        loop = channel._loop
        asyncio.set_event_loop(loop)
        asyncio.events.set_event_loop(loop)
        super().__init__(loop=loop, *args, **kwargs)
        self._loop = loop
        self._metadata = None
        self.blocksize = 2**18  # 2MB
        mirror_sync_methods(self)

    async def _rm_file(self, path, **kwargs):
        path = self._prefix_path(path)
        print(f"Deleting {path}")
        await self._client.Delete(DeleteRequest(key=Key(key=path)))

    async def _cp_file(self, path1, path2, **kwargs):
        # TODO: Implement cp in remote metadata store
        _, temp_filename = await aiofiles.tempfile.TemporaryFile(suffix=".yaml")
        await self._get_file(path1, temp_filename, **kwargs)
        await self._put_file(temp_filename, path2, **kwargs)

    async def _pipe_file(self, path, value, **kwargs):
        path = self._prefix_path(path)
        req = PutRequest(metadata=Metadata(), key=Key(key=path), object=Object(contents=value))
        try:
            await self._client.Put(req)
        except Exception as e:
            traceback.print_exception(e)
            raise

    async def _cat_file(self, path, start=None, end=None, **kwargs):
        path = self._prefix_path(path)
        req = GetRequest(key=Key(key=path))
        try:
            res: GetResponse = await self._client.Get(req)
            return res.object.contents
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                raise FileNotFoundError(path)
            traceback.print_exception(e)
            raise e
        except Exception as e:
            traceback.print_exception(e)
            raise

    async def _put_file(self, lpath, rpath, **kwargs):
        rpath = self._prefix_path(rpath)
        async with aiofiles.open(lpath, "rb") as f:
            await self._pipe_file(rpath, await f.read(), **kwargs)

    async def _get_file(self, rpath, lpath, **kwargs):
        rpath = self._prefix_path(rpath)
        async with aiofiles.open(lpath, "wb") as f:
            await f.write(await self._cat_file(rpath, **kwargs))

    async def _info(self, path, **kwargs):
        print(f"Info: {path}")
        path = self._prefix_path(path)
        req = HeadRequest(key=Key(key=path))
        try:
            res: HeadResponse = await self._client.Head(req)
        except grpc.RpcError as e:
            if e.code() == grpc.StatusCode.NOT_FOUND:
                raise FileNotFoundError(path)
            traceback.print_exception(e)
            raise e
        except Exception as e:
            traceback.print_exception(e)
            raise

        return {
            "size": res.size_bytes,
            "etag": res.etag,
            "tags": res.metadata.tag if res.metadata and res.metadata.tag else {},
            "type": "file",
        }

    async def _ls(self, path, detail=True, **kwargs):
        path = self._prefix_path(path)
        if not path.endswith("*"):
            path = f"{path}*"

        req = ListRequest(
            request=list_pb2.ListRequest(
                limit=1000,
                filters=[
                    list_pb2.Filter(
                        field="prefix",
                        function=list_pb2.Filter.GREATER_THAN_OR_EQUAL,
                        values=[path],
                    )
                ],
            )
        )

        keys = []
        while True:
            res: ListResponse = await self._client.List(req)
            keys.extend(res.keys)
            if not res.next_token or res.next_token == "":
                break

        return [
            {
                "name": k,
                "type": "file",
            }
            for k in keys
        ]


fsspec.register_implementation(name="unionmeta", cls=AsyncUnionMetaFS, clobber=True)



import base64
import sys
from dataclasses import dataclass
from functools import lru_cache
from importlib.metadata import version
from os import environ
from typing import Optional

from flyteidl.service.identity_pb2 import UserInfoRequest
from flyteidl.service.identity_pb2_grpc import IdentityServiceStub
from flytekit.clients.auth_helper import get_authenticated_channel
from flytekit.configuration import AuthType, Config, PlatformConfig, get_config_file

_GCP_SERVERLESS_ENDPOINT = "serverless-gcp.cloud-staging.union.ai"
_UNIONAI_SERVERLESS_API_KEY = "UNIONAI_SERVERLESS_API_KEY"
_SERVERLESS_IMAGE_REGISTRY: str = "us-central1-docker.pkg.dev/serverless-gcp-dataplane/union/unionai"


@dataclass
class _UnionAIConfig:
    serverless_endpoint: str = environ.get("UNIONAI_SERVERLESS_ENDPOINT", _GCP_SERVERLESS_ENDPOINT)
    org: Optional[str] = None
    config: Optional[str] = None


_UNIONAI_CONFIG = _UnionAIConfig()


@dataclass
class AppClientCredentials:
    endpoint: str
    client_id: str
    client_secret: str


def _encode_app_client_credentials(app_credentials: AppClientCredentials) -> str:
    """Encode app_credentials with base64."""
    data = f"{app_credentials.endpoint}:{app_credentials.client_id}:{app_credentials.client_secret}"
    return base64.b64encode(data.encode("utf-8")).decode("utf-8")


def _decode_app_client_credentials(encoded_str: str) -> AppClientCredentials:
    """Decode encoded base64 string into app credentials."""
    endpoint, client_id, client_secret = base64.b64decode(encoded_str.encode("utf-8")).decode("utf-8").split(":")
    return AppClientCredentials(endpoint=endpoint, client_id=client_id, client_secret=client_secret)


def _get_config_obj(config: Optional[str]) -> Config:
    """Get Config object."""
    if config is None:
        config = _UNIONAI_CONFIG.config

    cfg_file = get_config_file(config)
    if cfg_file is None:
        serverless_api_value = environ.get(_UNIONAI_SERVERLESS_API_KEY, None)
        config = Config.for_endpoint(endpoint=_UNIONAI_CONFIG.serverless_endpoint)

        if serverless_api_value is None:
            # Serverless clients points to the serverless endpoint by default.
            return config
        try:
            app_credentials = _decode_app_client_credentials(serverless_api_value)
        except Exception as e:
            raise ValueError(f"Unable to read {_UNIONAI_SERVERLESS_API_KEY}") from e

        return config.with_params(
            platform=PlatformConfig(
                endpoint=app_credentials.endpoint,
                client_id=app_credentials.client_id,
                client_credentials_secret=app_credentials.client_secret,
                auth_mode=AuthType.CLIENTSECRET,
            )
        )
    else:
        # Allow for --config to still be passed in for Managed+ users.
        return Config.auto(config)


def _get_organization(endpoint: str) -> str:
    """Get organization based on endpoint."""
    if _UNIONAI_CONFIG.org is not None:
        return _UNIONAI_CONFIG.org
    elif endpoint == _UNIONAI_CONFIG.serverless_endpoint:
        return _get_serverless_user_handle()
    else:
        # Managed+ users, the org is not required for requests and we set it ""
        # to replicate default flytekit behavior.
        return ""


@lru_cache
def _get_serverless_user_handle() -> str:
    """Get user_handle for serverless."""
    with get_authenticated_channel(
        PlatformConfig.for_endpoint(endpoint=_UNIONAI_CONFIG.serverless_endpoint),
    ) as channel:
        client = IdentityServiceStub(channel)
        user_info = client.UserInfo(UserInfoRequest())
        user_handle = user_info.additional_claims.fields["userhandle"]
        return user_handle.string_value


def _get_default_image() -> Optional[str]:
    """Get default image version."""
    cfg_obj = _get_config_obj(None)

    # TODO: This is only temporary to support GCP endpoints. When the unionai images are public,
    # we will always use unionai images
    if cfg_obj.platform.endpoint == _GCP_SERVERLESS_ENDPOINT:
        major, minor = sys.version_info.major, sys.version_info.minor
        unionai_version = version("unionai")
        if "dev" in unionai_version:
            suffix = "latest"
        else:
            suffix = unionai_version

        return f"{_SERVERLESS_IMAGE_REGISTRY}:py{major}.{minor}-{suffix}"

    # Returning None means that flytekit will use it's default images
    return None



import re
from typing import Optional

import rich_click as click
from flytekit.clients.auth_helper import get_authenticated_channel
from grpc import RpcError

from unionai._config import (
    _UNIONAI_CONFIG,
    AppClientCredentials,
    _encode_app_client_credentials,
    _get_config_obj,
    _get_organization,
    _get_serverless_user_handle,
)
from unionai.internal.identity.app_payload_pb2 import CreateAppRequest
from unionai.internal.identity.app_service_pb2_grpc import AppsServiceStub
from unionai.internal.identity.enums_pb2 import (
    ConsentMethod,
    GrantTypes,
    ResponseTypes,
    TokenEndpointAuthMethod,
)
from unionai.internal.secret.definition_pb2 import SecretIdentifier, SecretSpec
from unionai.internal.secret.payload_pb2 import CreateSecretRequest
from unionai.internal.secret.secret_pb2_grpc import SecretServiceStub


@click.group()
def create():
    """Create a resource."""


@create.command()
@click.pass_context
@click.argument("name")
@click.option("--value", prompt="Enter secret value", help="Secret value", hide_input=True)
@click.option("--project", help="Project name")
@click.option("--domain", help="Domain name")
def secret(
    ctx: click.Context,
    name: str,
    value: str,
    project: Optional[str],
    domain: Optional[str],
):
    """Create a secret with NAME."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    org = _get_organization(platform_obj.endpoint)

    channel = get_authenticated_channel(platform_obj)

    stub = SecretServiceStub(channel)
    request = CreateSecretRequest(
        id=SecretIdentifier(name=name, domain=domain, project=project, organization=org),
        secret_spec=SecretSpec(value=value),
    )
    try:
        stub.CreateSecret(request)
        click.echo(f"Created secret with name: {name}")
    except RpcError as e:
        raise click.ClickException(f"Unable to create secret with name: {name}\n{e}") from e


@create.command()
@click.pass_context
@click.argument("client_name")
def app(
    ctx: click.Context,
    client_name: str,
):
    """Create a app with CLIENT_NAME."""
    platform_obj = _get_config_obj(ctx.obj.get("config_file", None)).platform
    normalized_client_name = re.sub("[^0-9a-zA-Z]+", "-", client_name.lower())
    if platform_obj.endpoint == _UNIONAI_CONFIG.serverless_endpoint:
        userhandle = _get_serverless_user_handle()
        tenant = _UNIONAI_CONFIG.serverless_endpoint.split(".")[0]
        client_id = f"{tenant}-{userhandle}-{normalized_client_name}"
    else:
        client_id = normalized_client_name
    org = _get_organization(platform_obj.endpoint)

    channel = get_authenticated_channel(platform_obj)
    stub = AppsServiceStub(channel)
    request = CreateAppRequest(
        organization=org,
        client_id=client_id,
        client_name=client_id,
        grant_types=[GrantTypes.CLIENT_CREDENTIALS, GrantTypes.AUTHORIZATION_CODE],
        redirect_uris=["http://localhost:8080/authorization-code/callback"],
        response_types=[ResponseTypes.CODE],
        token_endpoint_auth_method=TokenEndpointAuthMethod.CLIENT_SECRET_BASIC,
        consent_method=ConsentMethod.CONSENT_METHOD_REQUIRED,
    )

    try:
        response = stub.Create(request)
    except RpcError as e:
        raise click.ClickException(f"Unable to create app with name: {client_name}\n{e}") from e

    click.echo(f"Client ID: {response.app.client_id}")
    click.echo("The following API key will only be shown once. Be sure to keep it safe!")
    click.echo("Configure your headless CLI by setting the following environment variable:")
    click.echo()

    serverless_api_key = _encode_app_client_credentials(
        AppClientCredentials(
            endpoint=platform_obj.endpoint,
            client_id=response.app.client_id,
            client_secret=response.app.client_secret,
        )
    )
    click.echo(f'export UNIONAI_SERVERLESS_API_KEY="{serverless_api_key}"')



# TODO: Remove try/except when https://github.com/flyteorg/flytekit/pull/2136/ is merged
try:
    import flytekit.core.artifact  # noqa: F401 place this line first

    from unionai.artifacts._artifact import Artifact
    from unionai.artifacts._triggers import Trigger

    __all__ = ["Artifact", "Trigger"]

except ModuleNotFoundError:
    pass



from datetime import timedelta
from typing import Any, Dict, List, Optional, Type, Union

import isodate
from flyteidl.core import identifier_pb2 as idl
from flyteidl.core import interface_pb2
from flytekit.core.artifact import Artifact, ArtifactQuery, Partition, TimePartition
from flytekit.core.context_manager import FlyteContextManager
from flytekit.core.launch_plan import LaunchPlan
from flytekit.core.tracker import TrackedInstance
from flytekit.core.type_engine import TypeEngine
from flytekit.core.workflow import WorkflowBase
from flytekit.models.interface import ParameterMap, Variable

from unionai.internal.artifacts import artifacts_pb2


class Trigger(TrackedInstance):
    """
     Trigger(
        trigger_on=[dailyArtifact, hourlyArtifact],
        inputs={
            "today_upstream": dailyArtifact,  # this means: use the matched artifact
            "yesterday_upstream": dailyArtifact.query(time_partition=dailyArtifact.time_partition - timedelta(days=1)),
            # this means: use the matched hourly artifact
            "other_daily_upstream": hourlyArtifact.query(partitions={"region": "LAX"}),
            "region": "SEA",  # static value that will be passed as input
            "other_artifact": UnrelatedArtifact.query(time_partition=dailyArtifact.time_partition - timedelta(days=1)),
            "other_artifact_2": UnrelatedArtifact.query(time_partition=hourlyArtifact.time_partition.truncate_to_day()),
            "other_artifact_3": UnrelatedArtifact.query(region=hourlyArtifact.time_partition.truncate_to_day()),
        },
    )
    """

    def __init__(
        self, trigger_on: List[Artifact], inputs: Optional[Dict[str, Union[Any, Artifact, ArtifactQuery]]] = None
    ):
        super().__init__()
        for t in trigger_on:
            if not isinstance(t, Artifact):
                raise ValueError("Trigger must be called with a list of artifacts")
        self.triggers = trigger_on
        self.inputs = inputs or {}  # user doesn't need to specify if the launch plan doesn't take inputs
        for k, v in self.inputs.items():
            if isinstance(v, ArtifactQuery):
                if v.bindings:
                    for b in v.bindings:
                        if b not in self.triggers:
                            raise ValueError(
                                f"Binding {b} id {id(b)} must reference a triggering artifact {self.triggers}"
                            )

            if isinstance(v, Artifact):
                if v not in self.triggers:
                    raise ValueError(f"Binding {v} id {id(v)} must reference a triggering artifact {self.triggers}")

    def get_parameter_map(
        self, input_python_interface: Dict[str, Type], input_typed_interface: Dict[str, Variable]
    ) -> interface_pb2.ParameterMap:
        """
        This is the key function that enables triggers to work. When declaring a trigger, the user specifies an input
        map in the form of artifacts, artifact time partitions, and artifact queries (possibly on unrelated artifacts).
        When it comes time to create the trigger, we need to convert all of these into a parameter map (because we've
        chosen Parameter as the method by which things like artifact queries are passed around). This function does
        that, and converts constants to Literals.
        """
        ctx = FlyteContextManager.current_context()
        pm = {}
        for k, v in self.inputs.items():
            var = input_typed_interface[k].to_flyte_idl()
            if isinstance(v, Artifact):
                aq = v.embed_as_query(self.triggers)
                p = interface_pb2.Parameter(var=var, artifact_query=aq)
            elif isinstance(v, ArtifactQuery):
                p = interface_pb2.Parameter(var=var, artifact_query=v.to_flyte_idl(bindings=self.triggers))
            elif isinstance(v, TimePartition):
                expr = None
                if v.op and v.other and isinstance(v.other, timedelta):
                    expr = str(v.op) + isodate.duration_isoformat(v.other)
                aq = v.reference_artifact.embed_as_query(self.triggers, bind_to_time_partition=True, expr=expr)
                p = interface_pb2.Parameter(var=var, artifact_query=aq)
            elif isinstance(v, Partition):
                # The reason is that if we bind to arbitrary partitions, we'll have to start keeping track of types
                # and if the experiment service is written in non-python, we'd have to reimplement a type engine in
                # the other language
                raise ValueError(
                    "Don't bind to non-time partitions. Time partitions are okay because of the known type."
                )
            else:
                lit = TypeEngine.to_literal(ctx, v, input_python_interface[k], input_typed_interface[k].type)
                p = interface_pb2.Parameter(var=var, default=lit.to_flyte_idl())

            pm[k] = p
        return interface_pb2.ParameterMap(parameters=pm)

    def to_flyte_idl(self) -> artifacts_pb2.Trigger:
        try:
            name = f"{self.instantiated_in}.{self.lhs}"
        except Exception:  # noqa broad for now given the changing nature of the tracker implementation.
            import random
            from uuid import UUID

            name = "trigger" + UUID(int=random.getrandbits(128)).hex

        # project/domain will be empty - to be bound later at registration time.
        artifact_ids = [a.to_id_idl() for a in self.triggers]

        return artifacts_pb2.Trigger(
            trigger_id=idl.Identifier(
                resource_type=idl.ResourceType.LAUNCH_PLAN,
                name=name,
            ),
            triggers=artifact_ids,
        )

    def __call__(self, *args, **kwargs):
        if len(kwargs) > 0:
            raise ValueError("Trigger does not support keyword arguments")
        if len(args) != 1:
            raise ValueError("Trigger must be called with a single function")

        entity = args[0]
        # This needs to create a Launch plan in the same
        if not isinstance(entity, WorkflowBase) and not isinstance(entity, LaunchPlan):
            raise ValueError("Trigger must be called with a workflow or launch plan")

        ctx = FlyteContextManager.current_context()

        pm = self.get_parameter_map(entity.python_interface.inputs, entity.interface.inputs)

        pm_model = ParameterMap.from_flyte_idl(pm)

        if isinstance(entity, LaunchPlan):
            raise NotImplementedError("Launch plan triggers are not yet supported")

        if isinstance(entity, WorkflowBase):
            default_lp = LaunchPlan.get_default_launch_plan(ctx, entity)
            trigger_lp = default_lp.clone_with(name=default_lp.name, parameters=pm_model)
            self_idl = self.to_flyte_idl()
            trigger_lp._additional_metadata = self_idl

            return entity



from typing import Optional

import click
from click import Group
from flytekit.remote import FlyteRemote

from unionai._config import _UNIONAI_CONFIG, _get_config_obj, _get_default_image
from unionai.cli._create import create
from unionai.cli._delete import delete
from unionai.cli._get import get
from unionai.cli._update import update
from unionai.remote import UnionRemote


class UnionAIPlugin:
    @staticmethod
    def get_remote(
        config: Optional[str],
        project: str,
        domain: str,
        data_upload_location: Optional[str] = None,
    ) -> FlyteRemote:
        cfg_obj = _get_config_obj(config)
        if cfg_obj.platform.endpoint == _UNIONAI_CONFIG.serverless_endpoint and project == "flytesnacks":
            project = "default"

        return UnionRemote(
            cfg_obj,
            default_project=project,
            default_domain=domain,
            data_upload_location=data_upload_location,
        )

    @staticmethod
    def configure_pyflyte_cli(main: Group) -> Group:
        """Configure pyflyte's CLI."""
        # Configure org at the top level:

        def _cli_main_config_callback(ctx, param, value):
            # Set org based on config from `pyflyte --config` for downstream cli
            # commands to use.
            _UNIONAI_CONFIG.config = value
            return value

        for p in main.params:
            if p.name == "config":
                p.callback = _cli_main_config_callback

        def _set_org(ctx, param, value):
            _UNIONAI_CONFIG.org = value

        main.params.append(
            click.Option(
                ["--org"],
                help="Set organization",
                hidden=True,
                callback=_set_org,
                expose_value=False,
            )
        )

        # NOTE: This is a stop-gap measure to make sure that workflows run on
        # serverless use the correct default project name. Eventually serverless
        # will support a "default" project instead of relying on
        # "{username}-project"
        from flytekit.clis.sdk_in_container.run import RunCommand, RunLevelParams

        class RunWithNewProjectDefault(RunCommand):
            def get_command(self, ctx, filename):
                command = super().get_command(ctx, filename)
                run_params: RunLevelParams = ctx.obj
                if run_params.is_remote and run_params.project == "flytesnacks":
                    cfg_obj = _get_config_obj(run_params.config_file)
                    if cfg_obj.platform.endpoint == _UNIONAI_CONFIG.serverless_endpoint:
                        # On Serverless, change the default project value
                        run_params.project = "default"
                return command

        main.add_command(RunWithNewProjectDefault(), "run")

        def update_or_create_group(new_group):
            try:
                main_group = main.commands[new_group.name]
                for name, command in new_group.commands.items():
                    main_group.add_command(command, name)
            except KeyError:
                main.add_command(new_group)

        new_groups = [create, delete, update, get]
        for group in new_groups:
            update_or_create_group(group)

        return main

    @staticmethod
    def secret_requires_group() -> bool:
        """Return True if secrets require group entry."""
        return False

    @staticmethod
    def get_default_image() -> Optional[str]:
        """Return default image."""
        return _get_default_image()



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from typing import ClassVar as _ClassVar

DESCRIPTOR: _descriptor.FileDescriptor

class ResponseTypes(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    CODE: _ClassVar[ResponseTypes]
    TOKEN: _ClassVar[ResponseTypes]
    ID_TOKEN: _ClassVar[ResponseTypes]

class ApplicationType(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    WEB: _ClassVar[ApplicationType]
    NATIVE: _ClassVar[ApplicationType]

class GrantTypes(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    AUTHORIZATION_CODE: _ClassVar[GrantTypes]
    IMPLICIT: _ClassVar[GrantTypes]
    PASSWORD: _ClassVar[GrantTypes]
    CLIENT_CREDENTIALS: _ClassVar[GrantTypes]
    REFRESH_TOKEN: _ClassVar[GrantTypes]
    DEVICE_CODE: _ClassVar[GrantTypes]

class TokenEndpointAuthMethod(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    NONE: _ClassVar[TokenEndpointAuthMethod]
    CLIENT_SECRET_POST: _ClassVar[TokenEndpointAuthMethod]
    CLIENT_SECRET_BASIC: _ClassVar[TokenEndpointAuthMethod]

class ConsentMethod(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    CONSENT_METHOD_NONE: _ClassVar[ConsentMethod]
    CONSENT_METHOD_TRUSTED: _ClassVar[ConsentMethod]
    CONSENT_METHOD_REQUIRED: _ClassVar[ConsentMethod]
CODE: ResponseTypes
TOKEN: ResponseTypes
ID_TOKEN: ResponseTypes
WEB: ApplicationType
NATIVE: ApplicationType
AUTHORIZATION_CODE: GrantTypes
IMPLICIT: GrantTypes
PASSWORD: GrantTypes
CLIENT_CREDENTIALS: GrantTypes
REFRESH_TOKEN: GrantTypes
DEVICE_CODE: GrantTypes
NONE: TokenEndpointAuthMethod
CLIENT_SECRET_POST: TokenEndpointAuthMethod
CLIENT_SECRET_BASIC: TokenEndpointAuthMethod
CONSENT_METHOD_NONE: ConsentMethod
CONSENT_METHOD_TRUSTED: ConsentMethod
CONSENT_METHOD_REQUIRED: ConsentMethod



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: flyteidl/artifacts/artifacts.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from flyteidl.admin import launch_plan_pb2 as flyteidl_dot_admin_dot_launch__plan__pb2
from flyteidl.core import literals_pb2 as flyteidl_dot_core_dot_literals__pb2
from flyteidl.core import types_pb2 as flyteidl_dot_core_dot_types__pb2
from flyteidl.core import identifier_pb2 as flyteidl_dot_core_dot_identifier__pb2
from flyteidl.core import artifact_id_pb2 as flyteidl_dot_core_dot_artifact__id__pb2
from flyteidl.core import interface_pb2 as flyteidl_dot_core_dot_interface__pb2
from flyteidl.event import cloudevents_pb2 as flyteidl_dot_event_dot_cloudevents__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"flyteidl/artifacts/artifacts.proto\x12\x11\x66lyteidl.artifact\x1a\x19google/protobuf/any.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a flyteidl/admin/launch_plan.proto\x1a\x1c\x66lyteidl/core/literals.proto\x1a\x19\x66lyteidl/core/types.proto\x1a\x1e\x66lyteidl/core/identifier.proto\x1a\x1f\x66lyteidl/core/artifact_id.proto\x1a\x1d\x66lyteidl/core/interface.proto\x1a flyteidl/event/cloudevents.proto\"\xca\x01\n\x08\x41rtifact\x12:\n\x0b\x61rtifact_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.ArtifactIDR\nartifactId\x12\x33\n\x04spec\x18\x02 \x01(\x0b\x32\x1f.flyteidl.artifact.ArtifactSpecR\x04spec\x12\x12\n\x04tags\x18\x03 \x03(\tR\x04tags\x12\x39\n\x06source\x18\x04 \x01(\x0b\x32!.flyteidl.artifact.ArtifactSourceR\x06source\"\xc7\x03\n\x15\x43reateArtifactRequest\x12=\n\x0c\x61rtifact_key\x18\x01 \x01(\x0b\x32\x1a.flyteidl.core.ArtifactKeyR\x0b\x61rtifactKey\x12\x18\n\x07version\x18\x03 \x01(\tR\x07version\x12\x33\n\x04spec\x18\x02 \x01(\x0b\x32\x1f.flyteidl.artifact.ArtifactSpecR\x04spec\x12X\n\npartitions\x18\x04 \x03(\x0b\x32\x38.flyteidl.artifact.CreateArtifactRequest.PartitionsEntryR\npartitions\x12L\n\x14time_partition_value\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x12timePartitionValue\x12\x39\n\x06source\x18\x06 \x01(\x0b\x32!.flyteidl.artifact.ArtifactSourceR\x06source\x1a=\n\x0fPartitionsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xfb\x01\n\x0e\x41rtifactSource\x12Y\n\x12workflow_execution\x18\x01 \x01(\x0b\x32*.flyteidl.core.WorkflowExecutionIdentifierR\x11workflowExecution\x12\x17\n\x07node_id\x18\x02 \x01(\tR\x06nodeId\x12\x32\n\x07task_id\x18\x03 \x01(\x0b\x32\x19.flyteidl.core.IdentifierR\x06taskId\x12#\n\rretry_attempt\x18\x04 \x01(\rR\x0cretryAttempt\x12\x1c\n\tprincipal\x18\x05 \x01(\tR\tprincipal\"\xd5\x02\n\x0c\x41rtifactSpec\x12,\n\x05value\x18\x01 \x01(\x0b\x32\x16.flyteidl.core.LiteralR\x05value\x12.\n\x04type\x18\x02 \x01(\x0b\x32\x1a.flyteidl.core.LiteralTypeR\x04type\x12+\n\x11short_description\x18\x03 \x01(\tR\x10shortDescription\x12\x39\n\ruser_metadata\x18\x04 \x01(\x0b\x32\x14.google.protobuf.AnyR\x0cuserMetadata\x12#\n\rmetadata_type\x18\x05 \x01(\tR\x0cmetadataType\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x1f\n\x0b\x66ile_format\x18\x07 \x01(\tR\nfileFormat\"z\n\x07Trigger\x12\x38\n\ntrigger_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.IdentifierR\ttriggerId\x12\x35\n\x08triggers\x18\x02 \x03(\x0b\x32\x19.flyteidl.core.ArtifactIDR\x08triggers\"Q\n\x16\x43reateArtifactResponse\x12\x37\n\x08\x61rtifact\x18\x01 \x01(\x0b\x32\x1b.flyteidl.artifact.ArtifactR\x08\x61rtifact\"b\n\x12GetArtifactRequest\x12\x32\n\x05query\x18\x01 \x01(\x0b\x32\x1c.flyteidl.core.ArtifactQueryR\x05query\x12\x18\n\x07\x64\x65tails\x18\x02 \x01(\x08R\x07\x64\x65tails\"N\n\x13GetArtifactResponse\x12\x37\n\x08\x61rtifact\x18\x01 \x01(\x0b\x32\x1b.flyteidl.artifact.ArtifactR\x08\x61rtifact\"`\n\rSearchOptions\x12+\n\x11strict_partitions\x18\x01 \x01(\x08R\x10strictPartitions\x12\"\n\rlatest_by_key\x18\x02 \x01(\x08R\x0blatestByKey\"\x80\x03\n\x16SearchArtifactsRequest\x12=\n\x0c\x61rtifact_key\x18\x01 \x01(\x0b\x32\x1a.flyteidl.core.ArtifactKeyR\x0b\x61rtifactKey\x12\x39\n\npartitions\x18\x02 \x01(\x0b\x32\x19.flyteidl.core.PartitionsR\npartitions\x12L\n\x14time_partition_value\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x12timePartitionValue\x12\x1c\n\tprincipal\x18\x04 \x01(\tR\tprincipal\x12\x18\n\x07version\x18\x05 \x01(\tR\x07version\x12:\n\x07options\x18\x06 \x01(\x0b\x32 .flyteidl.artifact.SearchOptionsR\x07options\x12\x14\n\x05token\x18\x07 \x01(\tR\x05token\x12\x14\n\x05limit\x18\x08 \x01(\x05R\x05limit\"j\n\x17SearchArtifactsResponse\x12\x39\n\tartifacts\x18\x01 \x03(\x0b\x32\x1b.flyteidl.artifact.ArtifactR\tartifacts\x12\x14\n\x05token\x18\x02 \x01(\tR\x05token\"\xdc\x01\n\x19\x46indByWorkflowExecRequest\x12\x43\n\x07\x65xec_id\x18\x01 \x01(\x0b\x32*.flyteidl.core.WorkflowExecutionIdentifierR\x06\x65xecId\x12T\n\tdirection\x18\x02 \x01(\x0e\x32\x36.flyteidl.artifact.FindByWorkflowExecRequest.DirectionR\tdirection\"$\n\tDirection\x12\n\n\x06INPUTS\x10\x00\x12\x0b\n\x07OUTPUTS\x10\x01\"\x7f\n\rAddTagRequest\x12:\n\x0b\x61rtifact_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.ArtifactIDR\nartifactId\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value\x12\x1c\n\toverwrite\x18\x03 \x01(\x08R\toverwrite\"\x10\n\x0e\x41\x64\x64TagResponse\"b\n\x14\x43reateTriggerRequest\x12J\n\x13trigger_launch_plan\x18\x01 \x01(\x0b\x32\x1a.flyteidl.admin.LaunchPlanR\x11triggerLaunchPlan\"\x17\n\x15\x43reateTriggerResponse\"T\n\x18\x44\x65\x61\x63tivateTriggerRequest\x12\x38\n\ntrigger_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.IdentifierR\ttriggerId\"\x1b\n\x19\x44\x65\x61\x63tivateTriggerResponse\"\x80\x01\n\x10\x41rtifactProducer\x12\x36\n\tentity_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.IdentifierR\x08\x65ntityId\x12\x34\n\x07outputs\x18\x02 \x01(\x0b\x32\x1a.flyteidl.core.VariableMapR\x07outputs\"\\\n\x17RegisterProducerRequest\x12\x41\n\tproducers\x18\x01 \x03(\x0b\x32#.flyteidl.artifact.ArtifactProducerR\tproducers\"\x7f\n\x10\x41rtifactConsumer\x12\x36\n\tentity_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.IdentifierR\x08\x65ntityId\x12\x33\n\x06inputs\x18\x02 \x01(\x0b\x32\x1b.flyteidl.core.ParameterMapR\x06inputs\"\\\n\x17RegisterConsumerRequest\x12\x41\n\tconsumers\x18\x01 \x03(\x0b\x32#.flyteidl.artifact.ArtifactConsumerR\tconsumers\"\x12\n\x10RegisterResponse\"\x9a\x01\n\x16\x45xecutionInputsRequest\x12M\n\x0c\x65xecution_id\x18\x01 \x01(\x0b\x32*.flyteidl.core.WorkflowExecutionIdentifierR\x0b\x65xecutionId\x12\x31\n\x06inputs\x18\x02 \x03(\x0b\x32\x19.flyteidl.core.ArtifactIDR\x06inputs\"\x19\n\x17\x45xecutionInputsResponse\"N\n\x10ListUsageRequest\x12:\n\x0b\x61rtifact_id\x18\x01 \x01(\x0b\x32\x19.flyteidl.core.ArtifactIDR\nartifactId\"_\n\x11ListUsageResponse\x12J\n\nexecutions\x18\x01 \x03(\x0b\x32*.flyteidl.core.WorkflowExecutionIdentifierR\nexecutions2\xfb\x0b\n\x10\x41rtifactRegistry\x12g\n\x0e\x43reateArtifact\x12(.flyteidl.artifact.CreateArtifactRequest\x1a).flyteidl.artifact.CreateArtifactResponse\"\x00\x12\x84\x01\n\x0bGetArtifact\x12%.flyteidl.artifact.GetArtifactRequest\x1a&.flyteidl.artifact.GetArtifactResponse\"&\x82\xd3\xe4\x93\x02 :\x01*\"\x1b/artifacts/api/v1/artifacts\x12\x8d\x01\n\x0fSearchArtifacts\x12).flyteidl.artifact.SearchArtifactsRequest\x1a*.flyteidl.artifact.SearchArtifactsResponse\"#\x82\xd3\xe4\x93\x02\x1d:\x01*\"\x18/artifacts/api/v1/search\x12\x64\n\rCreateTrigger\x12\'.flyteidl.artifact.CreateTriggerRequest\x1a(.flyteidl.artifact.CreateTriggerResponse\"\x00\x12\x9f\x01\n\x11\x44\x65\x61\x63tivateTrigger\x12+.flyteidl.artifact.DeactivateTriggerRequest\x1a,.flyteidl.artifact.DeactivateTriggerResponse\"/\x82\xd3\xe4\x93\x02):\x01*2$/artifacts/api/v1/trigger/deactivate\x12O\n\x06\x41\x64\x64Tag\x12 .flyteidl.artifact.AddTagRequest\x1a!.flyteidl.artifact.AddTagResponse\"\x00\x12\x65\n\x10RegisterProducer\x12*.flyteidl.artifact.RegisterProducerRequest\x1a#.flyteidl.artifact.RegisterResponse\"\x00\x12\x65\n\x10RegisterConsumer\x12*.flyteidl.artifact.RegisterConsumerRequest\x1a#.flyteidl.artifact.RegisterResponse\"\x00\x12m\n\x12SetExecutionInputs\x12).flyteidl.artifact.ExecutionInputsRequest\x1a*.flyteidl.artifact.ExecutionInputsResponse\"\x00\x12\xd8\x01\n\x12\x46indByWorkflowExec\x12,.flyteidl.artifact.FindByWorkflowExecRequest\x1a*.flyteidl.artifact.SearchArtifactsResponse\"h\x82\xd3\xe4\x93\x02\x62\x12`/artifacts/api/v1/search/execution/{exec_id.project}/{exec_id.domain}/{exec_id.name}/{direction}\x12\xf5\x01\n\tListUsage\x12#.flyteidl.artifact.ListUsageRequest\x1a$.flyteidl.artifact.ListUsageResponse\"\x9c\x01\x82\xd3\xe4\x93\x02\x95\x01\x12\x92\x01/artifacts/api/v1/usage/{artifact_id.artifact_key.project}/{artifact_id.artifact_key.domain}/{artifact_id.artifact_key.name}/{artifact_id.version}B\xcd\x01\n\x15\x63om.flyteidl.artifactB\x0e\x41rtifactsProtoP\x01Z?github.com/flyteorg/flyte/flyteidl/gen/pb-go/flyteidl/artifacts\xa2\x02\x03\x46\x41X\xaa\x02\x11\x46lyteidl.Artifact\xca\x02\x11\x46lyteidl\\Artifact\xe2\x02\x1d\x46lyteidl\\Artifact\\GPBMetadata\xea\x02\x12\x46lyteidl::Artifactb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'flyteidl.artifacts.artifacts_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\025com.flyteidl.artifactB\016ArtifactsProtoP\001Z?github.com/flyteorg/flyte/flyteidl/gen/pb-go/flyteidl/artifacts\242\002\003FAX\252\002\021Flyteidl.Artifact\312\002\021Flyteidl\\Artifact\342\002\035Flyteidl\\Artifact\\GPBMetadata\352\002\022Flyteidl::Artifact'
  _CREATEARTIFACTREQUEST_PARTITIONSENTRY._options = None
  _CREATEARTIFACTREQUEST_PARTITIONSENTRY._serialized_options = b'8\001'
  _ARTIFACTREGISTRY.methods_by_name['GetArtifact']._options = None
  _ARTIFACTREGISTRY.methods_by_name['GetArtifact']._serialized_options = b'\202\323\344\223\002 :\001*\"\033/artifacts/api/v1/artifacts'
  _ARTIFACTREGISTRY.methods_by_name['SearchArtifacts']._options = None
  _ARTIFACTREGISTRY.methods_by_name['SearchArtifacts']._serialized_options = b'\202\323\344\223\002\035:\001*\"\030/artifacts/api/v1/search'
  _ARTIFACTREGISTRY.methods_by_name['DeactivateTrigger']._options = None
  _ARTIFACTREGISTRY.methods_by_name['DeactivateTrigger']._serialized_options = b'\202\323\344\223\002):\001*2$/artifacts/api/v1/trigger/deactivate'
  _ARTIFACTREGISTRY.methods_by_name['FindByWorkflowExec']._options = None
  _ARTIFACTREGISTRY.methods_by_name['FindByWorkflowExec']._serialized_options = b'\202\323\344\223\002b\022`/artifacts/api/v1/search/execution/{exec_id.project}/{exec_id.domain}/{exec_id.name}/{direction}'
  _ARTIFACTREGISTRY.methods_by_name['ListUsage']._options = None
  _ARTIFACTREGISTRY.methods_by_name['ListUsage']._serialized_options = b'\202\323\344\223\002\225\001\022\222\001/artifacts/api/v1/usage/{artifact_id.artifact_key.project}/{artifact_id.artifact_key.domain}/{artifact_id.artifact_key.name}/{artifact_id.version}'
  _globals['_ARTIFACT']._serialized_start=369
  _globals['_ARTIFACT']._serialized_end=571
  _globals['_CREATEARTIFACTREQUEST']._serialized_start=574
  _globals['_CREATEARTIFACTREQUEST']._serialized_end=1029
  _globals['_CREATEARTIFACTREQUEST_PARTITIONSENTRY']._serialized_start=968
  _globals['_CREATEARTIFACTREQUEST_PARTITIONSENTRY']._serialized_end=1029
  _globals['_ARTIFACTSOURCE']._serialized_start=1032
  _globals['_ARTIFACTSOURCE']._serialized_end=1283
  _globals['_ARTIFACTSPEC']._serialized_start=1286
  _globals['_ARTIFACTSPEC']._serialized_end=1627
  _globals['_TRIGGER']._serialized_start=1629
  _globals['_TRIGGER']._serialized_end=1751
  _globals['_CREATEARTIFACTRESPONSE']._serialized_start=1753
  _globals['_CREATEARTIFACTRESPONSE']._serialized_end=1834
  _globals['_GETARTIFACTREQUEST']._serialized_start=1836
  _globals['_GETARTIFACTREQUEST']._serialized_end=1934
  _globals['_GETARTIFACTRESPONSE']._serialized_start=1936
  _globals['_GETARTIFACTRESPONSE']._serialized_end=2014
  _globals['_SEARCHOPTIONS']._serialized_start=2016
  _globals['_SEARCHOPTIONS']._serialized_end=2112
  _globals['_SEARCHARTIFACTSREQUEST']._serialized_start=2115
  _globals['_SEARCHARTIFACTSREQUEST']._serialized_end=2499
  _globals['_SEARCHARTIFACTSRESPONSE']._serialized_start=2501
  _globals['_SEARCHARTIFACTSRESPONSE']._serialized_end=2607
  _globals['_FINDBYWORKFLOWEXECREQUEST']._serialized_start=2610
  _globals['_FINDBYWORKFLOWEXECREQUEST']._serialized_end=2830
  _globals['_FINDBYWORKFLOWEXECREQUEST_DIRECTION']._serialized_start=2794
  _globals['_FINDBYWORKFLOWEXECREQUEST_DIRECTION']._serialized_end=2830
  _globals['_ADDTAGREQUEST']._serialized_start=2832
  _globals['_ADDTAGREQUEST']._serialized_end=2959
  _globals['_ADDTAGRESPONSE']._serialized_start=2961
  _globals['_ADDTAGRESPONSE']._serialized_end=2977
  _globals['_CREATETRIGGERREQUEST']._serialized_start=2979
  _globals['_CREATETRIGGERREQUEST']._serialized_end=3077
  _globals['_CREATETRIGGERRESPONSE']._serialized_start=3079
  _globals['_CREATETRIGGERRESPONSE']._serialized_end=3102
  _globals['_DEACTIVATETRIGGERREQUEST']._serialized_start=3104
  _globals['_DEACTIVATETRIGGERREQUEST']._serialized_end=3188
  _globals['_DEACTIVATETRIGGERRESPONSE']._serialized_start=3190
  _globals['_DEACTIVATETRIGGERRESPONSE']._serialized_end=3217
  _globals['_ARTIFACTPRODUCER']._serialized_start=3220
  _globals['_ARTIFACTPRODUCER']._serialized_end=3348
  _globals['_REGISTERPRODUCERREQUEST']._serialized_start=3350
  _globals['_REGISTERPRODUCERREQUEST']._serialized_end=3442
  _globals['_ARTIFACTCONSUMER']._serialized_start=3444
  _globals['_ARTIFACTCONSUMER']._serialized_end=3571
  _globals['_REGISTERCONSUMERREQUEST']._serialized_start=3573
  _globals['_REGISTERCONSUMERREQUEST']._serialized_end=3665
  _globals['_REGISTERRESPONSE']._serialized_start=3667
  _globals['_REGISTERRESPONSE']._serialized_end=3685
  _globals['_EXECUTIONINPUTSREQUEST']._serialized_start=3688
  _globals['_EXECUTIONINPUTSREQUEST']._serialized_end=3842
  _globals['_EXECUTIONINPUTSRESPONSE']._serialized_start=3844
  _globals['_EXECUTIONINPUTSRESPONSE']._serialized_end=3869
  _globals['_LISTUSAGEREQUEST']._serialized_start=3871
  _globals['_LISTUSAGEREQUEST']._serialized_end=3949
  _globals['_LISTUSAGERESPONSE']._serialized_start=3951
  _globals['_LISTUSAGERESPONSE']._serialized_end=4046
  _globals['_ARTIFACTREGISTRY']._serialized_start=4049
  _globals['_ARTIFACTREGISTRY']._serialized_end=5580
# @@protoc_insertion_point(module_scope)
